/* //////////////////////////////////////////////////////////////////////// */
/* // emdskagt.c ////////////////////////////////////////////////////////// */
/* //////////////////////////////////////////////////////////////////////// */

//
// Event Monitor Project
// version 0.6
//
// Juan Manuel Casillas
// assman@gsyc.inf.uc3m.es
//
//
// 
// emdiskagt [ Aka Event Monitor disk agent ]
//
// this program monitorize filesystems, and warns you if
// the free space fall down a specified value.
//
// this program is part of Event Monitor Project (EM), 
// a system to manage and operate a heterogeneous 
// network, based on message-passing.
//
// Usage:  emdiskagt &
//

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/vfs.h>
#include <string.h>
#include <strings.h>
#include <signal.h>


#include "libem.h"

  char SERVER[MAX_CAD];
  long PORT;   

/* //////////////////////////////////////////////////////////////////////// */
/* // load() ////////////////////////////////////////////////////////////// */
/* //////////////////////////////////////////////////////////////////////// */

void load(char *CF, char *SRV, long *PRT, struct fs_info fsdata[MAX_FS], int *num) {
FILE *f=NULL;
char cad[MAX_CAD];
float f1;
char s1[MAX_CAD];
     
f=fopen(CF,"r");
if (f==NULL) error(9,CF); 

*PRT=0;
bzero(SRV,MAX_CAD);

   read_cad(f,cad);
   if (sscanf(cad,"%s %ld",SRV,PRT)!=2) error(8,CF);
     
// now, read until end      

   bzero(cad,MAX_CAD);
   *num=0;  
   do {  
	read_cad(f,cad);	
        // printf(">>%s\n",cad);
	if (strlen(cad)==0) break;
	bzero(s1,MAX_CAD);
	if (sscanf(cad,"%s %f %s",fsdata[*num].name,&f1,s1)!=3)
	    error(8,CF);
	s2upper(s1);

	/* S1 can be ... Mb (MegaBytes) Gb (GigaBytes) */
	/* pass all the data to bytes */

	if (strcmp(s1,"MB")==0) {
	    fsdata[*num].level=(f1*1024*1024);
	    // printf("working with MB\n");
	}
	
	if (strcmp(s1,"GB")==0) {
	    fsdata[*num].level=(f1*1024*1024*1024);
	    // printf("working with GB\n");
         }		

	*num=*num+1;
	
   } while (strlen(cad)>0);
      
fclose(f);
}


/* //////////////////////////////////////////////////////////////////////// */
/* // exit_signal() /////////////////////////////////////////////////////// */
/* //////////////////////////////////////////////////////////////////////// */

void exit_signal(int i) {
  send_message(SERVER,PORT,0,"agentmgm:dsk","emdskagt stopped");
  signal(SIGINT,(void *)SIG_DFL); /* handle C-c */
  exit;
}

/* //////////////////////////////////////////////////////////////////////// */
/* // main() ////////////////////////////////////////////////////////////// */
/* //////////////////////////////////////////////////////////////////////// */

void main( int argc, char *argv[]) {
  struct fs_info fs_data[MAX_FS];
  struct statfs fs_tmp;
  float bused,bsize,bfree;
  float av_used, mb_free;
     int fs;
     char *ptr;
     char msg[MAX_CAD];
  char grp[MAX_CAD];
     int level;
  char *NFLOG;
  int n;
     
  signal(SIGINT,(void *)exit_signal); /* handle C-c */
  bzero(&fs_data,sizeof(fs_data));   
     
  NFLOG=getenv(ENV_VAR_DIR);
  if (NFLOG!=NULL) load(NFLOG,SERVER,&PORT,fs_data,&fs);
  else load(CFILE,SERVER,&PORT,fs_data,&fs);

  // show logo
 
 printf("[Event Monitor Proyect] Agent %s loggin on %s in port %ld\n",argv[0],SERVER,PORT); 

  send_message(SERVER,PORT,0,"agentmgm:disk","emdskagt started");
  // each FS_SLEEP_TIME , check the levels      
     
  while (1) {    
  for (n=0; n<fs; n++) {   
        // printf(">>%s,%.3f\n",fs_data[n].name,fs_data[n].level);
       
       statfs(fs_data[n].name,&fs_tmp);
       bsize=fs_tmp.f_bsize;
       bused=(fs_tmp.f_blocks-fs_tmp.f_bfree)*bsize; /* in bytes */
       bfree=(fs_tmp.f_bfree*bsize);                 /* in bytes */

       // printf("]]%.3f, %.3f\n",bfree,bused);
       
       if (bfree < fs_data[n].level ) {
	    
	    /* setup the level */
  	    /* very buggy */
	    
	    level=(int)((bfree*9)/fs_data[n].level);
	    
            // level=9;
	    /* setup the group */
	    
	    bzero(grp,MAX_CAD);
	    ptr=strstr(fs_data[n].name,"/");
	    ptr++;
	    strcat(grp,"disk:");
	    strcat(grp,ptr);
	    
	    /* setup the message */

	    av_used=(float)((fs_tmp.f_blocks-fs_tmp.f_bfree)*100)/
		(float)(fs_tmp.f_blocks);
	    mb_free=(float)bfree/(float)1024;
	    
	    bzero(msg,MAX_CAD);
	    sprintf(msg,"Disk %s is nearly full (%3.1f%% used, %3.1f Mb free)",
		     fs_data[n].name,av_used,mb_free);
	    
	    printf("%s %ld %d %s %s\n",SERVER,PORT,level,grp,msg);
	    send_message(SERVER,PORT,level,grp,msg);
	    
       }

  }
  sleep(FS_SLEEP_TIME);
  }
	
     
//  if (argc >=2 ) {
//	statfs(argv[1],&fs_stats);
//
//     printf("Statictics for %s\n",argv[1]);
//	printf("\t-Total Blocks:\t%ul\n",fs_stats.f_blocks);
//        printf("\t-Free Blocks:\t%d\n",fs_stats.f_bavail);
//        printf("\t-Block Size:\t%d\n",fs_stats.f_bsize);
//	printf("\t-Free Space:\t%d (MB)\n",fs_stats.f_bavail*fs_stats.f_bsize);
//        printf("\t-Used Space:\t%ul (MB)\n",
//	       (fs_stats.f_blocks-fs_stats.f_bfree)*fs_stats.f_bsize);
//  return 1;
//  }
//  return 0;

}
