# /////////////////////////////////////////////////////////////////////////////
# // mensajes.tcl /////////////////////////////////////////////////////////////
# /////////////////////////////////////////////////////////////////////////////

#
# //
# // Event Monitor Project
# // version 0.6
# //
# // Juan Manuel Casillas
# // assman@gsyc.inf.uc3m.es
# //
# //
# // Event Monitor Project
# //
# // mensajes.tcl
# //
# //
# // Manages all the messages (interface part) of
# // the system. Basically, using a widget and 
# // some procs.
# //
# // This program part of Event Monitor Project (EM), 
# // a system to manage and operate a heterogeneous 
# // network, based on message-passing.
# // See License file for details
# //
# // 
# // Sorry about the next documentation. I will translate
# // it as soon ass posible
# //
# //   
#
# Este fichero implementa un conjunto de funciones para implementar
# la ventana principal de la gestin de mensajes, es decir la ventana
# donde se almacenan los mensajes en forma de lineas. Utiliza las
# extensiones TiX
#
#
# Variables Globales
# ^^^^^^^^^^^^^^^^^^
#
#	Mensajes_Counter:		Identificador de row para la lista. Es 
#					unvoco y siempre crece.
#
#
# Procedimientos
# ^^^^^^^^^^^^^^
#
#	Mensajes:Create { w }		Crea una ventana w con el sistema
#					de gestion de mensajes. Es la 
#					primera funcion que deberia
#					invocarse
#
#	Mensajes:Show { w }		Muestra la ventana w. Debe de haberse
#					creado previamente para que funcione
#
#	Mensajes:Destroy { w }		Destruye la ventana w. Debe de haberse
#					creado previamente para que funcione
#
#	Mensajes:Insert { w cadena }	Inserta la cadena "cadena" en el 
#					sistema de mensajes. Debe de tener
#					al menos 6 campos, correctamente
#					introducidos. Debe de existir la
#					ventana w para que funcione
#
#
#

# ////////////////////////////////////////////////////////////////////////////
# // Variables Globales de la consola ////////////////////////////////////////
# ////////////////////////////////////////////////////////////////////////////


# ////////////////////////////////////////////////////////////////////////////
# // Mensajes:Count() ////////////////////////////////////////////////////////
# ////////////////////////////////////////////////////////////////////////////

proc Mensajes:Count { dato } {
    global STATUS_LINE
    set w .main.mensajes
    set hlist [$w.top.a subwidget hlist]
    set lista [ $hlist info selection ]
    set p [ llength $lista ]
    set STATUS_LINE "$p messages Selected"

}

# ////////////////////////////////////////////////////////////////////////////
# // Mensajes:Show() /////////////////////////////////////////////////////////
# ////////////////////////////////////////////////////////////////////////////

proc Mensajes:Show { wind } {
    upvar $wind w
    
    pack $w.top -side top -expand yes -fill both 
}

# ///////////////////////////////////////////////////////////////////////////
# // Mensajes:Background() //////////////////////////////////////////////////
# ///////////////////////////////////////////////////////////////////////////

proc Mensajes:Background { } {
 global New_Messages

 Dialog:New_Messages $New_Messages
 Mensajes:Refresh .main.mensajes

}

# ///////////////////////////////////////////////////////////////////////////
# // Mensajes:Refresh() /////////////////////////////////////////////////////
# ///////////////////////////////////////////////////////////////////////////

proc Mensajes:Refresh { w } {
  global MESSAGE_FILE
  global RFRESH_INTERVAL
  global EMTLOG_APP

  #
  # setup files
  #

  catch { exec $EMTLOG_APP }
  set hlist [$w.top.a subwidget hlist]    
  $hlist delete all
  set w .main.mensajes
  load_file w $MESSAGE_FILE
  after $RFRESH_INTERVAL "Mensajes:Background" 
    
}

# ////////////////////////////////////////////////////////////////////////////
# // Mensajes:Destroy() //////////////////////////////////////////////////////
# ////////////////////////////////////////////////////////////////////////////

proc Mensajes:Destroy { wind } {
    upvar $wind w
    destroy $w
}


# ////////////////////////////////////////////////////////////////////////////
# // Mensajes:AckAll() ///////////////////////////////////////////////////////
# ////////////////////////////////////////////////////////////////////////////

proc Mensajes:AckAll { w } {
    global EMTLOG_APP
    global STATUS_LINE
    global New_Messages
    global Msg_First
    global Msg_Last
   
    set hlist [$w.top.a subwidget hlist]
    catch { $hlist selection set $Msg_First $Msg_Last }
    set lista [ $hlist info selection ]
    set p [ llength $lista ]
    set n 0

    while { $n < $p } {
	set entr [ lindex $lista $n ]
	set valor [ $hlist entrycget $entr -data ]

	#
	# CALL EMTLOG HERE
	#
	catch { exec $EMTLOG_APP $valor }

	# catch { exec echo "1)$valor 2)$entr" >> /tmp/cositas }
	# catch { exec echo $datos >> /tmp/cositas }
        $hlist delete entry $entr

    incr n
    }
    
    set New_Messages [ expr { $New_Messages - $n } ]
    set STATUS_LINE "$n messages has been acknowledged"
}

# ////////////////////////////////////////////////////////////////////////////
# // Mensajes:Ack() //////////////////////////////////////////////////////////
# ////////////////////////////////////////////////////////////////////////////

proc Mensajes:Ack { w } {
    global EMTLOG_APP
    global New_Messages
    global STATUS_LINE

    set hlist [$w.top.a subwidget hlist]
    set lista [ $hlist info selection ]
    set p [ llength $lista ]
    set n 0

    while { $n < $p } {
	set entr [ lindex $lista $n ]
	set valor [ $hlist entrycget $entr -data ]

	#
	# CALL EMTLOG HERE
	#
	catch { exec $EMTLOG_APP $valor }

	# catch { exec echo "1)$valor 2)$entr" >> /tmp/cositas }
	# catch { exec echo $datos >> /tmp/cositas }
        $hlist delete entry $entr

    incr n
    }
    
    set New_Messages [ expr { $New_Messages - $n } ]
    set STATUS_LINE "$n messages has been acknowledged"
}

# ////////////////////////////////////////////////////////////////////////////
# // Mensajes:Insert() ///////////////////////////////////////////////////////
# ////////////////////////////////////////////////////////////////////////////

proc Mensajes:Insert { wind cadena } {
    upvar $wind w
    global Menssage_Counter	

    global MsgCol_bg0 MsgCol_bg1 MsgCol_bg2 MsgCol_bg3 MsgCol_bg4
    global MsgCol_bg5 MsgCol_bg6 MsgCol_bg7 MsgCol_bg8 MsgCol_bg9


    global Icon(level0) Icon(level1) Icon(level2) Icon(level3) Icon(level4)
    global Icon(level5) Icon(level6) Icon(level7) Icon(level8) Icon(level9)

    global icon_ghost icon_doc icon_windows icon_sh icon_exp
    global icon_happy icon_ray icon_xeyes icon_mah icon_clock

    set Icon(level0) $icon_mah
    set Icon(level1) $icon_sh
    set Icon(level2) $icon_happy
    set Icon(level3) $icon_ghost 
    set Icon(level4) $icon_exp
    set Icon(level5) $icon_windows
    set Icon(level6) $icon_doc
    set Icon(level7) $icon_xeyes
    set Icon(level8) $icon_clock
    set Icon(level9) $icon_ray


    set hlist [$w.top.a subwidget hlist]

    set style(level0) [tixDisplayStyle imagetext -refwindow $hlist \
           -fg #FFFFFF -padx 4 \
	   -bg $MsgCol_bg0 ]
	   
    set style(level1) [tixDisplayStyle imagetext -refwindow $hlist \
           -fg #FFFFFF -padx 4 \
	   -bg $MsgCol_bg1 ]

    set style(level2) [tixDisplayStyle imagetext -refwindow $hlist \
           -fg #FFFFFF -padx 4 \
	   -bg $MsgCol_bg2 ]

    set style(level3) [tixDisplayStyle imagetext -refwindow $hlist \
           -fg #FFFFFF -padx 4 \
	   -bg $MsgCol_bg3 ]

    set style(level4) [tixDisplayStyle imagetext -refwindow $hlist \
           -fg #FFFFFF -padx 4 \
	   -bg $MsgCol_bg4 ]

    set style(level5) [tixDisplayStyle imagetext -refwindow $hlist \
           -fg #FFFFFF -padx 4 \
	   -bg $MsgCol_bg5 ]

    set style(level6) [tixDisplayStyle imagetext -refwindow $hlist \
           -fg #FFFFFF -padx 4 \
	   -bg $MsgCol_bg6 ]

    set style(level7) [tixDisplayStyle imagetext -refwindow $hlist \
           -fg #FFFFFF -padx 4 \
	   -bg $MsgCol_bg7 ]

    set style(level8) [tixDisplayStyle imagetext -refwindow $hlist \
           -fg #FFFFFF -padx 4 \
	   -bg $MsgCol_bg8 ]

    set style(level9) [tixDisplayStyle imagetext -refwindow $hlist \
           -fg #FFFFFF -padx 4 \
	   -bg $MsgCol_bg9 ]


    set style(row) [tixDisplayStyle text -refwindow $hlist \
	-fg #FFFF00 -padx 4 \
	-bg #FF0000\
	-selectforeground  #FFFFFF\
	-selectbackground  #000000 ]

    set style(row2) [tixDisplayStyle text -refwindow $hlist \
	-fg black -padx 4 \
	-selectforeground  #FFFFFF\
	-selectbackground  #000000 ]

    set datos [ string trimright $cadena '\n' ]	
    set k [ split $datos " " ]
    set l [ llength $k ]

   # build the main fields --------------------------------------------------
   
    #
    # level
    # we need to change the color 
    # in function of the warning - level
    # 

   #
   # put some descriptive icons on the left ---------------------------------
   # I need to create the icons, of course ----------------------------------
   #
   
    set c1 [ lindex $k 1 ]
    set c2 [ lindex $k 0 ]
#   set row [$hlist add $Menssage_Counter -text $c1 -style $style(level$c1)]
    set row [ $hlist add $Menssage_Counter -itemtype imagetext  -text $c1 \
	   -image $Icon(level$c1) \
	   -data $c2 -style $style(level$c1)]
    
#-image [ tix getimage folder ]\


    # host
    set c1 [ lindex $k 2 ]
    $hlist item create $row 1 -itemtype text -text $c1 -style $style(row2)

    # date
    set c1 [ lindex $k 3 ]
    $hlist item create $row 2 -itemtype text -text $c1 -style $style(row2)

    # time
    set c1 [ lindex $k 4 ]
    $hlist item create $row 3 -itemtype text -text $c1 -style $style(row2)

    # group
    set c1 [ lindex $k 5 ]
    $hlist item create $row 4 -itemtype text -text $c1 -style $style(row2)
    
    # id
    set c1 [ lindex $k 0 ]
    $hlist item create $row 5 -itemtype text -text $c1 -style $style(row2)

   # build description -------------------------------------------------------

   set nm ""
   set p 6
   set l [ llength $k ]
   while { $p < $l } {
       lappend nm [ lindex $k $p ]
        incr p
   }

   $hlist item create $row 6 -itemtype text -text $nm -style $style(row2)
   incr Menssage_Counter
}	

# ////////////////////////////////////////////////////////////////////////////
# // Mensajes:Create() ///////////////////////////////////////////////////////
# ////////////////////////////////////////////////////////////////////////////

proc Mensajes:Create { wind } {
	upvar $wind w
	global FONT_HEADER

    #	
    # Creamos un frame y el widget ScrolledHList
    # en la ventana de dialogo
    #

    frame $w.top -relief flat -bd 1

    #
    # Creo una lista con un encabezamiento
    # y con colorines para que se vea bien 
    #
  
    tixScrolledHList $w.top.a -options {
	hlist.columns 8
	hlist.header  true
	hlist.relief ridge
	hlist.bd 1
	hlist.browseCmd Mensajes:Count
        hlist.wideSelection true
	hlist.selectBackground #202050
	hlist.selectForeground #F0F0F0
	hlist.selectMode extended
    }
    
    pack $w.top.a -expand yes -fill both -padx 1 -pady 1 -side top
 
    set hlist [$w.top.a subwidget hlist]

    #
    # Creo los titulos y sus estilos
    #	>> Notice that we have set the hlist.header subwidget option to true
    #      so that the header is displayed
    #

    # First some styles for the headers

    set style(Level) [tixDisplayStyle text -refwindow $hlist \
	-fg black -anchor c \
        -padx 20 -pady 2\
	-font $FONT_HEADER ]
	#-font [tix option get bold_font ]]

    set style(Host) [tixDisplayStyle text -refwindow $hlist \
	-fg black -anchor c \
        -padx 30 -pady 2\
	-font $FONT_HEADER ]

    set style(Date) [tixDisplayStyle text -refwindow $hlist \
	-fg black -anchor c \
        -padx 30 -pady 2\
	-font $FONT_HEADER ]

    set style(Time) [tixDisplayStyle text -refwindow $hlist \
	-fg black -anchor c \
        -padx 30 -pady 2\
	-font $FONT_HEADER ]

    set style(Group) [tixDisplayStyle text -refwindow $hlist \
	-fg black -anchor c \
        -padx 50 -pady 2\
	-font $FONT_HEADER ]

    set style(Id) [tixDisplayStyle text -refwindow $hlist \
	-fg black -anchor c \
        -padx 50 -pady 2\
	-font $FONT_HEADER ]

    set style(Description) [tixDisplayStyle text -refwindow $hlist \
	-fg black -anchor c \
        -padx 150 -pady 2\
	-font $FONT_HEADER ]

    $hlist header create 0 -itemtype text -text Level -style $style(Level)
    $hlist header create 1 -itemtype text -text Host -style $style(Host)
    $hlist header create 2 -itemtype text -text Date -style $style(Date)
    $hlist header create 3 -itemtype text -text Time -style $style(Time)
    $hlist header create 4 -itemtype text -text Group -style $style(Group)
    $hlist header create 5 -itemtype text -text ID -style $style(Id)
    $hlist header create 6 -itemtype text -text Description -style $style(Description)

    # Notice that we use 3 columns in the hlist widget. This way when the user
    # expands the windows wide, the right side of the header doesn't look
    # chopped off. The following line ensures that the 3 column header is
    # not shown unless the hlist window is wider than its contents.
    #
    $hlist column width 7 0

 }


